package error;

import static error.ErrorManager.Severity.*;

import java.io.PrintStream;
import java.text.MessageFormat;

import environment.RuntimeEnvironment;

/**
 * Prints error messages and potentially terminates the application.
 */
public class ErrorManager {

    private static final String ANSI_RESET = "\u001B[0m";
    private static final String ANSI_RED = "\u001B[31m";
    private static final String ANSI_PURPLE = "\u001B[35m";
    private static final String ANSI_YELLOW = "\u001B[33m";

    private RuntimeEnvironment environment;

    public ErrorManager() {
        this.environment = RuntimeEnvironment.getInstance();
    }

    public void handle(LispException lispException) {
        printMessage(lispException);

        if (isCritical(lispException))
            environment.terminateExceptionally();
    }

    private void printMessage(LispException lispException) {
        PrintStream output = selectOutputStream(lispException.getSeverity());
        output.println(formatMessage(lispException));
    }

    private PrintStream selectOutputStream(Severity severity) {
        if (severity == WARNING)
            return environment.getOutput();

        return environment.getErrorOutput();
    }

    private String formatMessage(LispException lispException) {
        Severity severity = lispException.getSeverity();
        String color = severity.getMessageColor();
        String prefix = severity.toDisplayString();

        return MessageFormat.format("{0}{1}: {2}{3}", color, prefix, lispException.getMessage(), ANSI_RESET);
    }

    private boolean isCritical(LispException lispException) {
        return lispException.getSeverity() == CRITICAL;
    }

    public static enum Severity {
        WARNING {

            public String getMessageColor() {
                return ANSI_YELLOW;
            }

            public String toDisplayString() {
                return "warning";
            }
        },
        ERROR {

            public String getMessageColor() {
                return ANSI_RED;
            }

            public String toDisplayString() {
                return "error";
            }
        },
        CRITICAL {

            public String getMessageColor() {
                return ANSI_PURPLE;
            }

            public String toDisplayString() {
                return "critical";
            }
        };

        public abstract String getMessageColor();

        public abstract String toDisplayString();
    }

}
